﻿<?php
// MegaLettering translated comics PHP engine
// Copyright (c) 2003-2008 Lapo Luchini <lapo@lapo.it>

// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

require_once('db.inc.php');
$style_cache = array();

function GetStyle($s) {
    global $dbprefix, $style_cache;
    if (isset($style_cache[$s]))
	return $style_cache[$s];
    // style was not cached, get it from the database
    $row = myqueryR("SELECT Font, Size FROM ${dbprefix}_Styles WHERE Name = ?;", array($s));
    if ($row) {
	$style = array(
	    'name' => $s,
	    'font' => "font/$row[0]",
	    'size' => (float)$row[1],
	);
	$bb = ImageTTFBBox($style['size'], 0, $style['font'], 'TOXtox');
	$style['top'] = $bb[5];
	$style['bottom'] = 1;
	$style_cache[$s] = $style;
    } else {
	error_log('Style "' . $s . '" not found in ' . __FILE__ . ':' . __LINE__);
	$style = FALSE;
    }
    return $style;
}

function MyImageTTFBBox_Piece($style, $piece) {
    $style = GetStyle($style);
    if (!file_exists($style['font'])) {
	error_log('Font "' . $style['font'] . '" not found in ' . __FILE__ . ':' . __LINE__);
	return FALSE;
    }
    $bb = ImageTTFBBox($style['size'], 0, $style['font'], $piece);
    return array(
	'left'   => $bb[0],
	'right'  => $bb[2],
	'width'  => $bb[2] - $bb[0],
	'top'    => $style['top'],
	'bottom' => $style['bottom'],
	'height' => $style['bottom'] - $style['top'],
    );
}

function StyleSplit($line, &$style, &$align) {
    $u = array();
    if (substr($line, 0, 8) == '$<align_') {
	switch(substr($line, 8, 1)) {
	case 'l':
	    $align = 'left';
	    break;
	case 'c':
	    $align = 'center';
	    break;
	case 'r':
	    $align = 'right';
	    break;
	}
	$line = substr($line, 10);
    }
    if (!preg_match_all('/\$<([a-z][a-z0-9_]*)(>)/', $line, $match, PREG_SET_ORDER | PREG_OFFSET_CAPTURE)) { // warning: PREG_OFFSET_CAPTURE requires php 4.3.0+
	$u[0] = array(
	    'style' => $style,
	    'text'  => $line,
	    'bbox'  => MyImageTTFBBox_Piece($style, $line),
	);
    } else {
	$last_idx = 0;
	$last_off = 0;
	foreach($match as $m) {
	    // $m[0] is the full match
	    // $m[1] is the first expression
	    // $m[x][0] is the string
	    // $m[x][1] is the offset
	    if ($m[1][1] - 2 - $last_off > 0) {
		if (($last_idx >= 2) && ($style == $u[$last_idx - 2])) {
		    $u[$last_idx - 1]['text'] .= substr($line, $last_off, $m[1][1] - 2 - $last_off);
		} else {
		    $text = substr($line, $last_off, $m[1][1] - 2 - $last_off);
		    $u[$last_idx] = array(
			'style' => $style,
			'text'  => $text,
			'bbox'  => MyImageTTFBBox_Piece($style, $text),
		    );
		    ++$last_idx;
		}
	    }
	    $style = $m[1][0];
	    $last_off = $m[2][1] + 1;
	}
	if ($last_off <= strlen($line)) {
	    if (($last_idx >= 2) && ($style == $u[$last_idx - 2])) {
		$u[$last_idx - 1]['text'] .= substr($line, $last_off);
	    } else {
		$text = substr($line, $last_off);
		$u[$last_idx] = array(
		    'style' => $style,
		    'text'  => $text,
		    'bbox'  => MyImageTTFBBox_Piece($style, $text),
		);
		++$last_idx;
	    }
	}
    }
    return $u;
}

function MyImageTTFBBox_Line($sline) {
    global $debug;
    $u = array(
	'left'   => 0,
	'right'  => 0,
	'width'  => 0,
	'top'    => 0,
	'bottom' => 0,
    );
    $first = TRUE;
    foreach($sline as $line) {
	$u['width'] += $line['bbox']['width'];
	if ($first) {
	    $u['top']    = $line['bbox']['top'];
	    $u['bottom'] = $line['bbox']['bottom'];
	    $u['left']   = $line['bbox']['left'];
	    $first = FALSE;
	} else {
	    if ($line['bbox']['top'] < $u['top'])
		$u['top'] = $line['bbox']['top'];
	    if ($line['bbox']['bottom'] > $u['bottom'])
		$u['bottom'] = $line['bbox']['bottom'];
	}
    }
    $u['right']  = $u['left'] + $u['width'];
    $u['height'] = $u['bottom'] - $u['top'];
    return $u;
}

function MyImageTTFBBox_Baloon($text, $style, &$align) {
    $text_style = array();
    foreach ($text as $line) { // calculates the total height
	$line_style = StyleSplit($line, $style, $align);
	$record = array(
	    'line' => $line_style,
	    'bbox' => MyImageTTFBBox_Line($line_style),
	);
	array_push($text_style, $record);
    }
    return $text_style;
}

function MyImageTTF_Baloon($img, $text, $style_default, $align_default, $centerX, $centerY, $colorFG, $colorMID) {
    global $debug;
    $align = $align_default;
    $text_style = MyImageTTFBBox_Baloon($text, $style_default, $align);
    $size_y = 0;
    foreach ($text_style as $line)
	$size_y += $line['bbox']['height'];
    $pos_y = $centerY - (int)($size_y / 2);
    foreach($text_style as $line) {
	switch($align) {
	case 'left':
	    $pos_x = $centerX;
	    break;
	case 'center':
	    $pos_x = $centerX - (int)($line['bbox']['width'] / 2);
	    break;
	case 'right':
	    $pos_x = $centerX - $line['bbox']['width'];
	    break;
	default:
	    die("unexpected alignment: [$line[align]]");
	}
	$pos_y += $line['bbox']['height'];
	if ($debug) ImageRectangle($img, $pos_x + $line['bbox']['left'], $pos_y + $line['bbox']['top'], $pos_x + $line['bbox']['right'], $pos_y + $line['bbox']['bottom'], $colorMID);
	foreach($line['line'] as $piece) {
	    $style_data = GetStyle($piece['style']);
	    if (!file_exists($style_data['font']))
		error_log('Font "' . $style_data['font'] . '" not found in ' . __FILE__ . ':' . __LINE__);
	    else
		ImageTTFText($img, $style_data['size'], 0, $pos_x, $pos_y, $colorFG, $style_data['font'], $piece['text']);
	    $pos_x += $piece['bbox']['width'];
	}
    }
}

function ImageCreateFromFile($s) {
    if (!preg_match('/\.(gif|jpe?g|png)$/i', strtolower($s), $m)) {
	error_log('No match in ' . __FILE__ . ':' . __LINE__);
	return FALSE;
    }
    switch ($m[1]{0}) {
    case 'p':
	return ImageCreateFromPNG($s);
    case 'g';
	return ImageCreateFromGIF($s);
    case 'j';
	return ImageCreateFromJPEG($s);
    }
}

function GetStripBaloons($id, $lang) {
    global $db, $dbprefix;
    $q = $db->prepare("
	SELECT
	    ${dbprefix}_Baloons.IDBaloon AS IDBaloon, ${dbprefix}_Baloons.Shape AS Shape,
	    ${dbprefix}_Baloons.ColorBG AS ColorBG, ${dbprefix}_Baloons.ColorFG AS ColorFG,
	    ${dbprefix}_Texts.CenterX AS CenterX, ${dbprefix}_Texts.CenterY AS CenterY,
	    ${dbprefix}_Baloons.Location AS Location, ${dbprefix}_Texts.Content AS Content
	FROM
	    ${dbprefix}_Baloons LEFT JOIN
	    ${dbprefix}_Texts ON
		${dbprefix}_Baloons.IDBaloon = ${dbprefix}_Texts.IDBaloon AND
		${dbprefix}_Texts.IDLanguage = ?
	WHERE
	    ${dbprefix}_Baloons.IDStrip = ?
	ORDER BY
	    ${dbprefix}_Baloons.Location, ${dbprefix}_Baloons.IDBaloon;
    ");
    if (!$q)
	return FALSE;
    $q->execute(array($lang, $id));
    return $q;
}

function SubtitleStrip($id, $lang, $dest) {
    global $db, $dbprefix, $debug;

    $style_default = myquery1("SELECT Default_Style FROM ${dbprefix}_Languages WHERE IDLanguage = ?;", array($lang)); 
    if ($style_default === FALSE)
	$style_default = 'normal';
    $align_default = 'center';

    if (function_exists('ImageColorClosestHWB'))
	$closest = 'ImageColorClosestHWB';
    elseif (function_exists('ImageColorClosest'))
	$closest = 'ImageColorClosest';
    else {
	error_log('No color function in ' . __FILE__ . ':' . __LINE__);
	return FALSE;
    }

    if (!$strip = GetStrip($id))
	return FALSE;
    if (!$img = ImageCreateFromFile($strip['filename'])) {
	error_log('Cannot create file in ' . __FILE__ . ':' . __LINE__);
	return FALSE;
    }

    // modifies the image
    $baloons = GetStripBaloons($id, $lang);
    foreach ($baloons as $row) {
	$shape = explode(',', $row['Shape']);
	$text = split("\r?\n", $row['Content']);

	// finds the colors
	$colorFG = $closest($img, ($row['ColorFG'] >> 16) & 0xFF, ($row['ColorFG'] >> 8) & 0xFF, $row['ColorFG'] & 0xFF);
	$colorBG = $closest($img, ($row['ColorBG'] >> 16) & 0xFF, ($row['ColorBG'] >> 8) & 0xFF, $row['ColorBG'] & 0xFF);
	if ($debug) $colorMID = $closest($img, (($row['ColorFG'] >> 17) & 0x7F) + (($row['ColorBG'] >> 17) & 0x7F), (($row['ColorFG'] >> 9) & 0x7F) + (($row['ColorBG'] >> 9) & 0x7F), (($row['ColorFG'] >> 1) & 0x7F) + (($row['ColorBG'] >> 1) & 0x7F));

	// blanks the baloon
	switch ($shape[0]) {
	case 'p': // polygon
	    $pts = array_slice($shape, 1);
	    ImageFilledPolygon($img, $pts, sizeof($pts)/2, $colorBG);
	    if($debug) ImagePolygon($img, $pts, sizeof($pts)/2, $colorMID);
	    break;
	case 'r': // rectangle
	    ImageFilledRectangle($img, $shape[1], $shape[2], $shape[3], $shape[4], $colorBG);
	    if($debug) ImageRectangle($img, $shape[1], $shape[2], $shape[3], $shape[4], $colorMID);
	    break;
	case 'e': // ellipse
	    ImageFilledEllipse($img, $shape[1], $shape[2], $shape[3], $shape[4], $colorBG);
	    if($debug) ImageEllipse($img, $shape[1], $shape[2], $shape[3], $shape[4], $colorMID);
	    break;
	default:
	    die("unknown baloon shape '$shape[0]' in " . __FILE__ . ':' . __LINE__);
	}

	if ($debug) {
	    ImageLine($img, $row['CenterX'] - 10, $row['CenterY'] - 10, $row['CenterX'] + 10, $row['CenterY'] + 10, $colorMID);
	    ImageLine($img, $row['CenterX'] + 10, $row['CenterY'] - 10, $row['CenterX'] - 10, $row['CenterY'] + 10, $colorMID);
	}

	// fills in the new text
	MyImageTTF_Baloon($img, $text, $style_default, $align_default, $row['CenterX'], $row['CenterY'], $colorFG, $debug ? $colorMID : $colorFG);
    }

    // streams out the resulting image
    ImagePNG($img, $dest, 9);

    // clears the buffers
    ImageDestroy($img);

    return TRUE;
}

function GetStrip($id, $lang = 'en') {
    global $dbprefix;
    $u = array(
	'id'       => (int)$id,
	'lang'     => $lang,
	'filename' => 'strips/' . substr("0000$id", -4) . '-' . $lang,
    );
    $row = myqueryR("SELECT URL, DimX, DimY FROM ${dbprefix}_Strips WHERE IDStrip = ?;", array($u['id']));
    if (!$row) {
	error_log("Empty result on strip ($id, $lang) in " . __FILE__ . ':' . __LINE__);
	return FALSE;
    }
    $url = (string)$row[0];
    $u['width'] = (int)$row[1];
    $u['height'] = (int)$row[2];
    $row = myqueryR("
	SELECT Content, Version FROM ${dbprefix}_Titles
	WHERE IDLanguage = ? AND IDStrip = ?;",
	array(($lang == 'xx') ? 'en' : $u['lang'], $u['id']));
    if (!$row) {
	error_log('Empty result in ' . __FILE__ . ':' . __LINE__);
	return FALSE;
    }
    $u['title'] = (string)$row[0];
    $version = $row[1] ? strtotime($row[1]) : 0;
    if ($u['lang'] == 'en') {
	//TODO: check $version here, also
	if (!preg_match('/\.(gif|jpe?g|png)$/i', $url, $m)) {
	    error_log('No match in ' . __FILE__ . ':' . __LINE__);
	    return FALSE;
	}
	$m[1] = strtolower($m[1]);
	$u['mime'] = 'image/png';
	if ($m[1] == 'gif')
	    $u['mime'] = 'image/gif';
	elseif (($m[1] == 'jpg') || ($m[1] == 'jpeg'))
	    $u['mime'] = 'image/jpeg';
	$u['filename'] .=  '.' . strtolower($m[1]);
	if (!file_exists($u['filename'])) {
	    if (!@copy($url, $u['filename'])) {
		error_log('Cannot copy from "' . $url . '" in ' . __FILE__ . ':' . __LINE__);
		return FALSE;
	    }
	    ClearStatCache();
	}
    } else {
	$u['filename'] .= '.png';
	$u['mime'] = 'image/png';
	if (file_exists($u['filename']) && ($version > filemtime($u['filename'])))
	    unlink($u['filename']);
	if (!file_exists($u['filename'])) {
	    if (!SubtitleStrip($u['id'], $u['lang'], $u['filename']))
		return FALSE;
	    ClearStatCache();
	}
    }
    // now that we have the image locally, we can find its dimensions if they were not already known
    if (($u['width'] == 0 ) || ($u['height'] == 0 )) {
	// notice: this means that all the languages *must* have the same size
	$real_dim = getimagesize($u['filename']);
	$u['width'] = (int)$real_dim[0];
	$u['height'] = (int)$real_dim[1];
	myupdate("UPDATE ${dbprefix}_Strips SET DimX = ?, DimY = ? WHERE IDStrip = ?;",
	    array($u['width'], $u['height'], $u['id']));
    }
    return $u;
}

function GetStripTitle($id, $lang = 'en') {
    global $dbprefix;
    return myquery1("SELECT Content FROM ${dbprefix}_Titles WHERE IDLanguage = ? AND IDStrip = ?;",
	array($lang, $id));
}

function DeleteStrip($id, $lang = 'en') {
    global $dbprefix;
    $filename = 'strips/' . substr("0000$id", -4) . '-' . $lang;
    if ($lang == 'en') {
	$url = myquery1("SELECT URL FROM ${dbprefix}_Strips WHERE IDStrip = ?;", array($id));
	if (!preg_match('/\.(gif|jpe?g|png)$/i', $url, $m)) {
	    error_log('No match in ' . __FILE__ . ':' . __LINE__);
	    return FALSE;
	}
	$filename .=  '.' . strtolower($m[1]);
    } else
	$filename .= '.png';
    if (file_exists($filename))
	unlink($filename);
    ClearStatCache();
    return TRUE;
}

/*
function SecureDownload($file) {
    global $config;
    if (!$config['SecureDownload'])
	return $file;
    if (!preg_match('|^/|', $file))
	$file = "/$file";
    // time with 256-seconds resolution, so the image is cached if page is visited within that time
    $t = sprintf('%08x', time() & (~0xFF));
    return sprintf('/sec/%s/%s%s',
	md5($config['SessionKey'] . $file . $t),
	$t,
	$file
    );
}
*/

function SecureDownload($file) {
    if (!preg_match('|^/|', $file))
	$file = "/$file";
    return $file;
}
?>
