<?php
/**
* Cache class
* 
* Cache gestisce localmente la cache di files e documenti
* generati dinamicamente, in modo da minimizzare i tempi
* di creazione delle pagine.
* 
* @package Cache
* @author  Alex Dupre <sysadmin@alexdupre.com>
* @version $Id: Cache.php,v 1.4 2002/07/07 16:37:43 alexdupre Exp $
*/
class Cache
{
    /**
    * Directory dove salvare i files di cache
    * 
    * @access private
    * @var string $path
    * @see Cache()
    */
    var $path;
    
    /**
    * Class constructor
    * 
    * Inizializza la directory in cui operare.
    * 
    * @param string $path path della directory
    */
    function Cache($path)
    {
        $this->path = $path;
    }
    
    /**
    * Converte un array associativo in una stringa utilizzabile in un filename
    * 
    * La funzione, dopo avere ordinato l'array per chiave,
    * genera una stringa unendo tutti i campi dell'array
    * e restituisce l'hash MD5 di quest'ultima.
    * 
    * @access private
    * @param  array   $vars array con le coppie nome/valore usate per la generazione della stringa
    * @return string  stringa utilizzabile come estensione del file di cache
    */
    function encode($vars)
    {
        ksort($vars);
        foreach ($vars as $name => $val) $tmp[] = "$name=$val";
        return md5(join('&', $tmp));
    }
    
    /**
    * Recupera un documento dalla cache
    * 
    * Se il file  piu' vecchio del numero di secondi specificati nella variabile
    * $time, la cache non verr considerata attendibile. I rimanenti tre
    * parametri vengono utilizzati per ricreare il nome del file contenente il
    * documento cachato.
    * 
    * @access public
    * @param  integer $time   tempo per cui  da considerarsi valido il documento cachato (in secondi)
    * @param  string  $object parola che identifica univocamente il modulo o l'oggetto chiamante
    * @param  string  $action parola che identifica l'azione o il documento richiesto dal chiamante
    * @param  array   $vars   array con i parametri usati per discriminare pi versioni dello stesso documento
    * @return string  il documento cachato o FALSE se non  stato trovato
    */
    function load($time, $object, $action, $vars = array())
    {
	$name = $this->path . $object;
	if(strlen($action) > 0)
	    $name .= '-' . $action;
	if(count($vars) > 0)
	    $name .= '.' . $this->encode($vars);
        $cf = new File($name);
        if ($cf->mtime() + $time < time()) return FALSE;
        return $cf->safeRead();
    }
    
    /**
    * Inserisce/aggiorna un documento nella cache
    * 
    * @access public
    * @param  string  $doc    documento da cachare
    * @param  string  $object parola che identifica univocamente il modulo o l'oggetto chiamante
    * @param  string  $action parola che identifica l'azione o il documento creato dal chiamante
    * @param  array   $vars   array con i parametri usati per discriminare pi versioni dello stesso documento
    * @return boolean TRUE se l'inserimento/aggiornamento ha avuto successo, FALSE altrimenti
    */
    function save($doc, $object, $action, $vars = array())
    {
	$name = $this->path . $object;
	if(strlen($action) > 0)
	    $name .= '-' . $action;
	if(count($vars) > 0)
	    $name .= '.' . $this->encode($vars);
        $cf = new File($name);
        return $cf->safeWrite($doc);
    }
}
?>
