/*
 * Created on 17-giu-2005
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package it.megatokyo;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.awt.image.MultiPixelPackedSampleModel;

import javax.swing.JComponent;

public class LetteredImage extends JComponent implements MouseListener,
        MouseMotionListener {

    /** Immagine da letterare */
    protected BufferedImage image;

    /** Poligono da sbiancare */
    protected SmartPolygon poly;

    /** Colore con cui sbiancare */
    protected Color fill;

    /** Quanti per cui ingrandire */
    protected int zoom;

    /** Dimensione immagine */
    transient private java.awt.Dimension imageSize;

    /** Cache delle coordinate zoomate del poligono */
    transient int xcache[], ycache[];

    /** Cache dell'oggetto filtro sull'immagine */
    transient BufferedImageOp filter;

    public LetteredImage(BufferedImage img, Color fill, int zoom) {
        System.err.println("Using image: " + img);
        System.err.println("Sample model: " + img.getSampleModel());
        image = img;
        poly = new SmartPolygon();
        this.fill = fill;
        this.zoom = zoom;
        // if (image.getType() == BufferedImage.TYPE_BYTE_BINARY) {
        if (image.getSampleModel() instanceof MultiPixelPackedSampleModel) {
            // it seems that BytePackedRaster such as the one used by
            // BufferedImage.TYPE_BYTE_BINARY can't be resized on the fly and
            // takes up much more memory than this conversion does
            // http://archives.java.sun.com/cgi-bin/wa?A2=ind9901&L=java2d-interest&D=0&P=1302
            ColorConvertOp c = new ColorConvertOp(null);
            // here we assume that 1,2,4 bpp images only are grey
            // TODO: find a way to contert from BytePackedRaster to
            // ByteInterleavedRaster without changing ColorSpace
            image = new BufferedImage(img.getWidth(), img.getHeight(),
                    BufferedImage.TYPE_BYTE_GRAY);
            // --
            // image = new BufferedImage(img.getColorModel(),
            // Raster.createWritableRaster(
            // new SinglePixelPackedSampleModel(DataBuffer.TYPE_BYTE,
            // img.getWidth(), img.getHeight(), new int[] {15}), new Point(0,
            // 0)), true,
            // null);
            // --
            // image = new BufferedImage(img.getWidth(), img.getHeight(),
            // BufferedImage.TYPE_BYTE_INDEXED, new IndexColorModel(8, /*
            // * TODO: come dannaz' si prende la palette di un'immagine
            // * esistente?
            // */));
            c.filter(img, image);
            System.err.println("Converted to: " + image);
        }
        imageSize = new Dimension(img.getWidth() * zoom, img.getHeight() * zoom);
        filter = new AffineTransformOp(AffineTransform.getScaleInstance(zoom,
                zoom), AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
        updateCache();
        addMouseListener(this);
        addMouseMotionListener(this);
    }

    /**
     * @return Returns the zoom.
     */
    public int getZoom() {
        return zoom;
    }

    /**
     * @param zoom
     *            The zoom to set.
     */
    public void setZoom(int zoom) {
        if ((zoom > 0) && (this.zoom != zoom)) {
            this.zoom = zoom;
            imageSize.width = image.getWidth() * zoom;
            imageSize.height = image.getHeight() * zoom;
            filter = new AffineTransformOp(AffineTransform.getScaleInstance(
                    zoom, zoom), AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
            updateCache();
            repaint();
        }
    }

    public Dimension getMinimumSize() {
        return imageSize;
    }

    public Dimension getPreferredSize() {
        return imageSize;
    }

    private void updateCache() {
        if ((xcache == null) || (xcache.length != poly.xpoints.length)) {
            xcache = new int[poly.xpoints.length];
            ycache = new int[poly.xpoints.length];
            for (int n = 0; n < poly.npoints; ++n) {
                xcache[n] = poly.xpoints[n] * zoom;
                ycache[n] = poly.ypoints[n] * zoom;
            }
        }
    }

    /**
     * @see java.awt.Component#paint(java.awt.Graphics)
     */
    @Override
    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.drawImage(image, filter, 0, 0);
        g2.setColor(fill);
        if (poly.npoints >= 3)
            g2.fillPolygon(xcache, ycache, poly.npoints);
    }

    public void mouseClicked(MouseEvent e) { /* empty */
    }

    public void mousePressed(MouseEvent e) {
        Rectangle r;
        if ((e.getModifiersEx() & MouseEvent.SHIFT_DOWN_MASK) != 0) {
            r = poly.getBounds();
            poly.removePoint(e.getX() / zoom, e.getY() / zoom);
        } else if (e.getButton() == MouseEvent.BUTTON1) {
            poly.addPoint(e.getX() / zoom, e.getY() / zoom);
            r = poly.getBounds();
        } else {
            r = poly.getBounds();
            poly.movePoint(e.getX() / zoom, e.getY() / zoom);
            r = poly.getBounds().union(r);
        }
        updateCache();
        repaint(r.x * zoom, r.y * zoom, r.width * zoom, r.height * zoom);
        e.consume();
    }

    public void mouseReleased(MouseEvent e) { /* empty */
    }

    public void mouseEntered(MouseEvent e) { /* empty */
    }

    public void mouseExited(MouseEvent e) { /* empty */
    }

    public void mouseDragged(MouseEvent e) {
        if ((e.getModifiersEx() & MouseEvent.SHIFT_DOWN_MASK) == 0) {
            Rectangle r = poly.getBounds();
            poly.movePoint(e.getX() / zoom, e.getY() / zoom);
            r = poly.getBounds().union(r);
            updateCache();
            repaint(r.x * zoom, r.y * zoom, r.width * zoom, r.height * zoom);
            e.consume();
        }
    }

    public void mouseMoved(MouseEvent e) { /* empty */
    }

}
