/*
 * Created on 17-giu-2005
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package it.megatokyo;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

public class SmartPolygon extends Polygon {

    /**
     * 
     */
    public SmartPolygon() {
        super();
    }

    /**
     * @param xpoints
     * @param ypoints
     * @param npoints
     */
    public SmartPolygon(int[] xpoints, int[] ypoints, int npoints) {
        super(xpoints, ypoints, npoints);
    }

    /**
     * Finds the nearest point to the one specified.
     * 
     * @param x
     *            the specified x coordinate
     * @param y
     *            the specified y coordinate
     * @return index of nearest point
     */
    private int nearestPoint(int x, int y) {
        int nearest = -1;
        double distance = Double.POSITIVE_INFINITY;
        for (int n = 0; n < npoints; ++n) {
            double d2 = Point2D.distanceSq(xpoints[n], ypoints[n], x, y);
            if (d2 < distance) {
                nearest = n;
                distance = d2;
            }
        }
        return nearest;
    }

    /**
     * Adds the specified point splitting the nearest segment.
     * 
     * @param x
     *            the specified x coordinate
     * @param y
     *            the specified y coordinate
     * @see java.awt.Polygon#addPoint(int, int)
     */
    @Override
    public void addPoint(int x, int y) {
        super.addPoint(x, y);
        // beginning with the fourth point, we need to check order
        if (npoints > 3) {
            int nearest = -1;
            double distance = Double.POSITIVE_INFINITY;
            for (int n = 0, prec = npoints - 2; n < npoints - 1; ++n) {
                double d2 = Line2D.ptSegDistSq(xpoints[n], ypoints[n],
                        xpoints[prec], ypoints[prec], x, y);
                if (d2 < distance) {
                    nearest = n;
                    distance = d2;
                }
                prec = n;
            }
            if (nearest != -1) {
                for (int n = npoints - 1; n > nearest; --n) {
                    xpoints[n] = xpoints[n - 1];
                    ypoints[n] = ypoints[n - 1];
                }
                xpoints[nearest] = x;
                ypoints[nearest] = y;
            }
        }
    }

    /**
     * Moves the nearest point to the specified coordinates.
     * 
     * @param x
     *            the specified x coordinate
     * @param y
     *            the specified y coordinate
     */
    public void movePoint(int x, int y) {
        int nearest = nearestPoint(x, y);
        if (nearest != -1) {
            xpoints[nearest] = x;
            ypoints[nearest] = y;
            invalidate();
        }

    }

    /**
     * Removes the nearest point to the specified coordinates.
     * 
     * @param x
     *            the specified x coordinate
     * @param y
     *            the specified y coordinate
     */
    public void removePoint(int x, int y) {
        int nearest = nearestPoint(x, y);
        if (nearest != -1) {
            for(int n = nearest + 1; n < npoints; ++n) {
                xpoints[n - 1] = xpoints[n];
                ypoints[n - 1] = ypoints[n];
            }
            --npoints;
            invalidate();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer u = new StringBuffer("Poly[");
        for (int n = 0; n < npoints; ++n) {
            u.append(xpoints[n]);
            u.append(',');
            u.append(ypoints[n]);
            u.append('-');
        }
        u.append(']');
        return u.toString();
    }

}