<?PHP
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Stephan Schmidt <schst@php-tools.net>                       |
// |          Lapo Luchini <lapo@lapo.it>                                 |
// +----------------------------------------------------------------------+

/**
 * This is a basic example of how to use HTTP_Server to build
 * a custom webserver.
 * 
 * @category    HTTP
 * @package     HTTP_Server
 * @subpackage  Examples
 * @author      Stephan Schmidt <schst@php-tools.net>
 */

require_once 'HTTP/Server.php';

/**
 * This is a basic example of how to use HTTP_Server to build
 * a custom webserver.
 * 
 * @category    HTTP
 * @package     HTTP_Server
 * @subpackage  Examples
 * @author      Stephan Schmidt <schst@php-tools.net>
 */
class HTTP_Server_PHP extends HTTP_Server {

   /**
    * document root
    * @var string $documentRoot
    */
    var $documentRoot   =   ".";

   /**
    * handle a GET request
    * this method should return an array of the following format:
    *
    * array(
    *        "code"    => 200,
    *        "headers" => array(
    *                            "Content-Type" => "text/plain",
    *                            "X-Powered-By" => "PEAR"
    *                          ),
    *        "body"    => "This is the actual response"
    *      )
    *
    * @access public
    * @param  integer $clientId   id of the client that send the headers
    * @param  array   $headers    request headers
    * @return mixed   $response   array containing the response or false if the method handles the response
    * @see    POST()
    */
    function GET($clientId, &$request)
    {
	$path_info       = $request->getPathInfo();
	$path_translated = realpath($this->documentRoot . $path_info);

	$headers = array();

	//  path does not exist
	if (!file_exists($path_translated))
	    return array("code" => 404);

	// path is a directory
	if (is_dir($path_translated)) {
	    $headers['Content-Type'] = 'text/html';
	    $body = 
		"<html>\n" .
		" <head><title>Directory listing of $path_info</title></head>\n" .
		" <body>\n" .
		"  <h1>Directory listing of $path_info</h1>\n" .
		"  <hr />\n";
	    $files = $this->_getFilesInDir($path_translated);
	    if (substr($path_info, -1) != "/") {
		$path_info = $path_info."/";
	    }
	    foreach ($files as $file)
		$body .= "<a href=\"$path_info$file\">$file</a><br/>";
	    $body .=
		" </body>\n" .
		"</html>\n";
	}
	// path is a file
	elseif (is_readable($path_translated) ) {
	    // path is a PHP script
	    if(preg_match('/\.php$/', $path_translated)) {
		$this->_driver->sendData($clientId, "HTTP/1.0 200 $_statusCodes[200]\n");
		foreach($this->_defaultResponseHeaders as $header => $value)
		    $this->_driver->sendData($clientId, "$header: $value\n");
		$this->PHP_CGI($clientId, $path_translated, $request);
		return NULL;
	    } else {
		$body = fopen($path_translated, "rb");
	    }
	} else
	    return array("code" => 403);

	return array(
	    "code"    => 200,
	    "headers" => $headers,
	    "body"    => $body,
	);
    }

    function POST($clientId, &$request) {
	return $this->GET($clientId, $request);
    }
    
    function PHP_CGI($clientId, $script, &$request) {
	$descriptorspec = array(
	    0 => array("pipe", "r"),  // stdin is a pipe that the child will read from
	    1 => array("pipe", "w"),  // stdout is a pipe that the child will write to
	    //2 => array("file", "/tmp/error-output.txt", "a") // stderr is a file to write to
	);
	putenv('SERVER_PROTOCOL=' . $request->protocol);
	putenv('REMOTE_ADDR=127.0.0.1'); //TODO mettere quello vero
	putenv('SCRIPT_FILENAME=' . $script);
	putenv('QUERY_STRING=' . $request->query_string);
	putenv('REQUEST_METHOD=POST');
	putenv('CONTENT_TYPE=application/x-www-form-urlencoded');
	putenv('CONTENT_LENGTH=' . strlen($request->post));
	if(isset($request->headers['cookie']))
	    putenv('HTTP_COOKIE=' . $request->headers['cookie']);
	$process = proc_open('php-cgi -c php.ini', $descriptorspec, $pipes);
	if (is_resource($process)) {
	    fwrite($pipes[0], $request->post);
	    fclose($pipes[0]);
	    while (!feof($pipes[1]))
	       $this->_driver->sendData($clientId, fgets($pipes[1], 10240));
	    fclose($pipes[1]);
	    $return_value = proc_close($process);
	} else
	    $return_value = false;
	return $return_value;
    }

   /**
    * read all files in a directory
    *
    * @param string $dir directory
    * @return array $files
    */
    function _getFilesInDir($dir)
    {
	$files = array();
	if (!is_dir($dir)) {
	    return $files;
	}
	
	$dh = dir( $dir );
	while ($entry = $dh->read()) {
	    if ($entry == "." || $entry == "..") {
		continue;
	    }
	    if (is_readable($dir."/".$entry)) {
		array_push($files,$entry);
	    }
	}
	return $files;
    }
}
    //  instantiate the server
//    $myServer = &new HTTP_Server_PHP('127.0.0.1',80);

    /**
     * use this on machines that do not have ext/pcntl compiled
     */
    $myServer = &new HTTP_Server_PHP('127.0.0.1', 80, 'Sequential');

    //$myServer->_driver->setDebugMode(true);
    $myServer->_driver->setDebugMode(false);
    //$myServer->documentRoot = dirname( __FILE__ );
    $myServer->documentRoot = '..';
    $myServer->start();
?>
