﻿<?php
// MegaLettering translated comics PHP engine
// Copyright (c) 2003-2008 Lapo Luchini <lapo@lapo.it>

// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

ini_set('zlib.output_compression', TRUE);

require_once('XML/Serializer.php');
require_once('image.inc.php');
require_once('gettext.inc.php');
require_once('rewrite.inc.php');

$tickertitle = 'MegaTokyo (' . strtoupper($lang) . ')';

// exceptions for languages that have its own DNS entry
switch ($lang) {
case 'de':
    $base = 'http://www.megatokyo.de';
    break;
case 'fr':
    $base = 'http://www.megatokyo.fr';
    break;
case 'nl':
    $base = 'http://www.megatokyo.nl';
    break;
default:
    $base = 'http://www.megatokyo.it';
}

$options = array(
    'indent'    => '    ',
    'linebreak' => "\n",
    'typeHints' => false,
    'addDecl'   => true,
    'encoding'  => 'UTF-8',
    'rootName'  => 'rss',
    'rootAttributes' => array(
	'version' => '2.0',
	'xmlns:atom' => 'http://www.w3.org/2005/Atom'
    ),
    'mode' => 'simplexml',
    'attributesArray' => 'ATTR',
);

setlocale(LC_TIME, 'en_US.US-ASCII');
$q = $db->prepare("
    SELECT
	Translated.IDStrip,
	Translated.Content AS Title,
	Original.Publish AS PublishOriginal,
	Translated.Publish AS PublishTranslated
    FROM
	${dbprefix}_Titles AS Translated LEFT JOIN
	${dbprefix}_Titles AS Original ON Translated.IDStrip = Original.IDStrip
    WHERE
	Translated.IDLanguage = '$lang' AND
	Original.IDLanguage = 'en' AND
	Translated.Publish <= NOW()
    ORDER BY
	Translated.Publish DESC, Translated.IDStrip DESC
    LIMIT 20;
");
$q->execute();
$q->setFetchMode(PDO::FETCH_ASSOC);
while ($row = $q->fetch()) {
    $q2 = $db->prepare("
	SELECT DISTINCT
	    ${dbprefix}_Characters.Name AS Name
	FROM
	    ${dbprefix}_Baloons JOIN
	    ${dbprefix}_Characters ON ${dbprefix}_Baloons.IDCharacter = ${dbprefix}_Characters.IDCharacter
	WHERE
	    ${dbprefix}_Baloons.IDStrip = ? AND ${dbprefix}_Characters.Thread = 1
	ORDER BY
	    ${dbprefix}_Characters.Name;");
    $q2->execute(array($row['IDStrip']));
    $chars = $q2->fetchAll(PDO::FETCH_COLUMN, 1);
    $q2->closeCursor();

    $strip = GetStrip($row['IDStrip'], $lang);
    $title = htmlspecialchars(sprintf(_('strip #%1$d: %2$s'), $row['IDStrip'], $row['Title']));
    $html = '<p>' . sprintf(
	    _('A new strip with title “%1$s” was published, featuring characters such as: %2$s'),
	    $row['Title'],
	    implode(', ', array_map('_', $chars))
	) . ".</p>\n" .
	'<img src="' . $base . SecureDownload($strip['filename']) . '" width="' . $strip['width'] . '" height="' . $strip['height'] . '" border="1" title="' . $title . '" alt="' . $title . '"/>';

    $item[] = array(
	'guid' => array(
	    'ATTR' => array('isPermaLink' => 'false'),
	    0 => sprintf('%s%04d', $lang, $row['IDStrip'])
	),
	'title' => sprintf(
	    _('[%1$04d] %2$s (original: %3$s, translation: %4$s)'),
	    $row['IDStrip'],
	    $row['Title'],
	    $row['PublishOriginal'],
	    $row['PublishTranslated']
	),
	'link'        => preg_replace('/[\x{7F}-\x{FF}]/e', 'sprintf("%%%02X", ord("$0"))', $base . GetURL($row['IDStrip'], $lang, $row['Title'])),
	'pubDate'     => gmdate('D, d M Y H:i:s \G\M\T', strtotime($row['PublishTranslated'])),
	'description' => $html
    );
}
$q->closeCursor();

$data['channel'] = array(
    'title' => $tickertitle,
    'description' => _('MegaTokyo - relax, we understand j00'),
    'link'  => $base,
    'language' => $lang,
    'atom:link' => array('ATTR' => array(
	'rel' => 'self',
	'type' => 'application/rss+xml',
	'href' => "$base/newsfeed.php"
    )),
    'item' => $item
);

$serializer = new XML_Serializer($options);

if ($serializer->serialize($data)) {
    header('Content-type: text/xml;charset=' . $options['encoding']);
    echo $serializer->getSerializedData();
}
?>
