﻿<?php
//TODO add encryption options?

/**
 * Encodes any PHP data type in a safe envelope.
 * Safe meaning that any transmission error or alteration will be detected.
 * @param data any PHP data type
 * @param base64 true for ASCII output, false for binary output
 * @return safeguarded envelope
 */
function safe_encode($data, $base64 = false) {
    // serialize any PHP data type into a string
    $data = serialize($data);
    // bzcompress the string to avoid excessive redundancy
    $data = bzcompress($data);
    // add a header with the hash of the payload (hash used is SHAd-256)
    $data = mhash(MHASH_SHA256, mhash(MHASH_SHA256, $data)) . $data;
    // base64_encode if requested
    if ($base64)
	$data = chunk_split(base64_encode($data));
    // return the safeguarded envelope
    return $data;
}

/**
 * Decodes a safe envelope containing any PHP data.
 * Any transmission error or alteration of even a single bit will be detected.
 * @param data envelope to open
 * @param base64 true for ASCII envelope, false for binary envelope
 * @return null if envelope is invalid or tampered, else contained PHP data
 */
function safe_decode($data, $base64 = false) {
    // base64_decode if requested
    if ($base64)
	$data = base64_decode($data);
    // separate header from payload
    $hash = substr($data, 0, 32);
    $data = substr($data, 32);
    // check the hash of the payload against the one advertised in header
    if (mhash(MHASH_SHA256, mhash(MHASH_SHA256, $data)) != $hash)
	return null;
    // if the hash is correct neither bzdecompress nor unserialize should find
    // any error at all
    return unserialize(bzdecompress($data));
}
?>
