﻿<?php
include('header.inc.php');

require_once('lib/camaleont/framework/Filesystem/File.php');
require_once('lib/camaleont/framework/Cache/Cache.php');

// receive GET parameters
$month = 0;
if (isset($_GET['month']) && is_numeric($_GET['month']))
    $month = (int)$_GET['month'];
$log = 0;
if (isset($_GET['log']) && ($_GET['log'] === '1'))
    $log = 1;

$cache = new Cache('cache/');
if ($load = $cache->load(600 /* secs */, 'stripstat', 'html', array('lang' => $lang, 'month' => $month, 'log' => $log))) {
    echo $load;
} else {
ob_start();

function DaysInMonth($month) {
    $y = 2000 + (int)($month / 100);
    $m = $month % 100;
    $now = getdate();
    if (($y == $now['year']) && ($m == $now['mon']))
	return $now['mday'];
    switch($m) {
	case  1: return 31;
	case  2: return ((($y % 4 == 0) && ($y % 100 != 0)) || ($y % 400 == 0)) ? 29 : 28;
	case  3: return 31;
	case  4: return 30;
	case  5: return 31;
	case  6: return 30;
	case  7: return 31;
	case  8: return 31;
	case  9: return 30;
	case 10: return 31;
	case 11: return 30;
	case 12: return 31;
	default: return 0;
    }
}

function DynamicPlaces($f) {
    if (abs($f) < 10)
	return sprintf('%.2f', $f);
    if (abs($f) < 100)
	return sprintf('%.1f', $f);
    return sprintf('%.0f', $f);
}

// this query is quite heavy on the DB
$table = array('T' => array('T' => 0));
if ($load = $cache->load(600 /* secs */, 'stripstat', 'data', array('month' => $month)))
    $table = unserialize($load);
else {
    if ($month > 0) {
	$ty = 2000 + (int)($month / 100);
	$tm = $month % 100;
	$q = $db->prepare("
	    SELECT IDLanguage, DAY(Date) AS Day, SUM(Count)
	    FROM MTi_Stats
	    WHERE YEAR(Date) = ? AND MONTH(Date) = ?
	    GROUP BY IDLanguage, Day
	    ORDER BY IDLanguage, Day;");
	$q->execute(array($ty, $tm));
	unset($ty, $tm);
    } else  {
	$q = $db->prepare('
	    SELECT IDLanguage, (YEAR(Date) - 2000) * 100 + MONTH(Date) AS Month, SUM(Count)
	    FROM MTi_Stats
	    GROUP BY IDLanguage, Month
	    ORDER BY IDLanguage, Month;');
	$q->execute();
    }
    $q->setFetchMode(PDO::FETCH_NUM);
    while ($row = $q->fetch()) {
	$row_lang = (string)$row[0];
	$row_date = (int)$row[1];
	$row_count = (int)$row[2];
	if (!isset($table[$row_lang]))
	    $table[$row_lang] = array();
	if (!isset($table['T'][$row_date]))
	    $table['T'][$row_date] = 0;
	if (!isset($table[$row_lang]['T']))
	    $table[$row_lang]['T'] = 0;
	if (!isset($table['T'][$row_date]))
	    $table['T'][$row_date] = 0;
	if (!isset($table[$row_lang][$row_date]))
	    $table[$row_lang][$row_date] = 0;
	$table['T']['T'] += $row_count;
	$table[$row_lang]['T'] += $row_count;
	$table['T'][$row_date] += $row_count;
	$table[$row_lang][$row_date] += $row_count;
    }
    $q->closeCursor();
    unset($row, $row_lang, $row_date, $row_count, $q);
    $cache->save(serialize($table), 'stripstat', 'data', array('month' => $month));
}

if ($month == 0) {
    $tot_numdays = 0;
    foreach($table['T'] as $i_m => $i_c)
	$tot_numdays += DaysInMonth($i_m);
}

// extract list of languages from the array itself
$table_langs = array_keys($table);

// let's sort things in a sensible way
function HigherTotal($a, $b) {
    global $table;
    if ($table[$a]['T'] == $table[$b]['T'])
	return 0;
    return ($table[$a]['T'] < $table[$b]['T']) ? +1 : -1;
}
usort($table_langs, 'HigherTotal');
ksort($table['T']); // should already be sorted from the query, but just to be sure

// create the graph
require_once('lib/jpgraph/jpgraph.php');
require_once('lib/jpgraph/jpgraph_line.php');
if ($log)
    require_once('lib/jpgraph/jpgraph_log.php');

$graph_image = $log ? "stripstat-$month-log.png" : "stripstat-$month-lin.png";
$graph = new Graph(650, 300, $graph_image, -1, false);
$graph->img->SetMargin(40, 20, 20, 20);
$graph->img->SetAntiAliasing();
$graph->SetScale($log ? 'textlog' : 'textlin');
$graph->xaxis->SetFont(FF_ARIAL, FS_NORMAL, 8);
$graph->xaxis->SetLabelAngle(60);
$graph->yaxis->SetFont(FF_ARIAL, FS_NORMAL, 8);
$graph->legend->SetFont(FF_ARIAL, FS_NORMAL, 8);
$graph->legend->SetAbsPos(60, 30, 'left', 'top');

$colors = array(
    'red',      'blue',      'green',
    'orange',   'cyan',      'magenta',
    'lightred', 'lightblue', 'lightgreen',
    'darkred',  'darkblue',  'darkgreen',
);
$color_i = 0;
foreach($table_langs as $table_lang)
    if ($table_lang != 'T') {
	$datay = array();
	foreach($table['T'] as $date => $date_total)
	    if ($date != 'T')
		$datay[] = isset($table[$table_lang][$date])
		    ? ($month == 0)
			? $table[$table_lang][$date] / DaysInMonth($date)
			: $table[$table_lang][$date]
		    : 0.0;
	$p = new LinePlot($datay);
	$p->SetLegend($table_lang);
	$p->SetColor($colors[$color_i++]);
	$p->SetCenter();
	$graph->Add($p);
    }

$graph->Stroke();
?>
<br/>
<img src="cache/<?=$graph_image?>" width="650" height="300"/><br/>
<?=_('switch to')?>: <a href="?lang=<?=$lang?>&amp;month=<?=$month?>&amp;log=<?=1-$log?>"><?=$log ? _('linear graph') : _('logarithmic graph')?></a><br/>
<br/>
<table width="700" border="0" bgcolor="#000000" cellspacing="0" cellpadding="0" summary="<?=_('stats')?>">
  <tr>
    <td width="700" valign="center" align="center">
      <img src="/image/pix.gif" width="1" height="10" border="0" alt=""/>
      <table width="676" border="0" bgcolor="#23272b" cellspacing="2" cellpadding="2">
	<tr>
	  <td width="675" valign="top" align="left">
	    <font class="atso"><b>s t a t s</b></font> <br/>
	    <table border="0" bgcolor="#23272b">
<?
$_total = _('Total');
echo "	      <tr align=\"right\">\n";
echo "		<td></td>\n";
foreach($table_langs as $table_lang)
    echo '		<td><b>' . ($table_lang == 'T' ? $_total : $table_lang) . "</b></td>\n";
echo "	      </tr>\n";
foreach($table['T'] as $date => $date_total) {
    echo "	      <tr align=\"right\">\n";
    if ($month > 0)
	$date_str = iconv('ISO-8859-1', 'UTF-8', strftime('%A&nbsp;%Y-%m-%d', strtotime((2000 + (int)($month / 100)) . '-' . ($month % 100) . '-' .$date)));
    else
	$date_str = 
	    "<a href=\"?lang=$lang&amp;month=$date&amp;log=$log\">" .
	    iconv('ISO-8859-1', 'UTF-8', strftime('%Y&nbsp;%B', strtotime((2000 + (int)($date / 100)) . '-' . ($date % 100) . '-1'))) .
	    '</a>';
    echo '		<td><b>' . ($date == 'T' ? ($month > 0 ? "<a href=\"?lang=$lang&amp;month=0&amp;log=$log\">$_total</a>" : $_total) : $date_str) . "</b></td>\n";
    foreach($table_langs as $table_lang)
	if ($month > 0)
	    echo '		<td>' . (isset($table[$table_lang][$date]) ? $table[$table_lang][$date] : 0) . "</td>\n";
	else
	    echo '		<td>' . (isset($table[$table_lang][$date]) ? DynamicPlaces($table[$table_lang][$date] / ($date == 'T' ? $tot_numdays : DaysInMonth($date))) : 0) . "</td>\n";
    echo "	      </tr>\n";
}
?>
	    </table>
	  </td>
	</tr>
      </table>
      <img src="/image/pix.gif" width="1" height="10" border="0" alt=""/>
    </td>
  </tr>
</table>
<br/>
<?
$cache->save(ob_get_contents(), 'stripstat', 'html', array('lang' => $lang, 'month' => $month, 'log' => $log));
ob_end_flush();
}

include('footer.inc.php');
?>
